# --------------------------------------------------------------------------
###
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# --------------------------------------------------------------------------
# $Id: SymLock.pm,v 1.2 2001/06/23 01:34:14 anil Exp $
# $Name:  $
# --------------------------------------------------------------------------

package SymLock;
#flocks files

use strict;
use Fcntl ':flock'; # import LOCK_* constants
use Symbol;
use File::Basename;

my $LOCK_PATH = "/etc/virtualhosting/locks/";
my $SYMLOCK_NB = 1;

sub get_process_start_time($) {

    my $pid = shift;
    if (! open (PID, "</proc/".$pid."/stat")) {
	# TODO Check error code, etc... Log the error
	return 0;
    }
    my $line = <PID>;
    my @values = split(/ /, $line);
    my $time = $values[21];
    close (PID);

    return $time;
}

sub lock ($) {
    my ($file, $opts) = @_;
    if (! defined ($opts)) {
	$opts = 0;
    }

    my $lock_file = $LOCK_PATH.$file;
    my $lock_dir = dirname $lock_file;
    my $stime = get_process_start_time($$);
    my $tmp_file = "$lock_file.$$.$stime";

    while (1) {

	# TODO Check the directory write permissions here

	# Open the directory. Remember to close the var whenever returning
	if (!open(LOCKDIR, "<$lock_dir")) {
	    # TODO log error
	    return 0;
	}

	# Try to get the upper level lock on the directory
	if ( !flock(LOCKDIR, (LOCK_EX | LOCK_NB))) {
	    close(LOCKDIR);
	    if ($opts & $SYMLOCK_NB) {
		sleep 2;
		next;
	    } else {
		# TODO log error
		return 0;
	    }
	}

	# If the file exists, then determine the owner
	if (-f $lock_file) {
	    my $lock_link = readlink $lock_file;
	    my $basename = basename $lock_link;
	    my ($lbname, $lpid, $lstime) = split ('\.', $basename);
	    my $lstarttime = get_process_start_time($lpid);

	    if (!$lstarttime or ($lstarttime ne $lstime)) {
		# owner is dead and gone
		unlink $lock_file;
		unlink $lock_link;
	    } elsif ($opts & $SYMLOCK_NB) {
		# owner exists and we don't want to wait
		close(LOCKDIR);
		return 0;
	    } else {
		# owner exists and we're willing to wait
		close(LOCKDIR);
		sleep 2;
		next;
	    }
	}

	else {
	    # The lock file may exist as a broken symlink
	    lstat $lock_file;
	    unlink $lock_file;
	}

	# Check if the tmp_file exists
	if (-f $tmp_file) {
	    print STDERR "tmp_file exists... lock usage error? $tmp_file\n";
	    close(LOCKDIR);
	    return 0;
	}
	else {
	    if (!open(TMPFILE, ">$tmp_file")) {
		close(LOCKDIR);
		return 0;
	    }
	}
	close(TMPFILE);

	if (! symlink($tmp_file, $lock_file)) {
	    # TODO log error
	    unlink $tmp_file;
	    close(LOCKDIR);
	    return 0;
	}

	# Finally got the lock
	close(LOCKDIR);
	return 1;
    }

}

sub unlock ($) {
    my ($filename) = @_;
    my $lock_file = $LOCK_PATH.$filename;
    my $lock_dir = dirname $lock_file;
    my $stime = get_process_start_time($$);
    my $tmp_file = "$lock_file.$$.$stime";

    while (1) {

	# TODO Check the directory permissions before

	# Open the directory as a file here to be able to obtain the flock
	if ( !open (LOCKDIR, "<$lock_dir") ) {
	    # TODO log error
	    return 0;
	}

	if ( !flock(LOCKDIR, LOCK_EX) ) {
	    close(LOCKDIR);
	    sleep 2;
	    next;
	}

	if (-f $lock_file) {
	    unlink $lock_file;
	    unlink $tmp_file;
	}

	close(LOCKDIR);
	return 1;
    }

}

#
# return 1, if the lock exists
# return 0, if it doesn't
#
sub Check($) {
    my ($filename) = @_;
    my $lock_file = $LOCK_PATH.$filename;

    if (! -f $lock_file) {
	return 0;
    }
    return 1;
}


return 1;
